import { AlarmConfigService } from '@business/services/http/alarm-config.service';
import { SensorsService } from '@business/services/http/sensors.service';
import { Router } from '@angular/router';
import { ModalHelper, _HttpClient } from '@delon/theme';
import { NzModalService, NzMessageService } from 'ng-zorro-antd';
import { OrganizationService } from '@business/services/http/organization.service';
import { Grid, Column, PageBean, ResultBean } from '@business/entity/grid';
import { Organization, OrganizationSensorUnit } from '@business/entity/data';
import { Component, OnInit } from '@angular/core';
import { Subject } from 'rxjs/Subject';
import { Types } from '@business/enum/types.enum';
import { OrganizationEditComponent } from 'app/routes/systems/organization/organization-edit/organization-edit.component';
import { zip } from 'rxjs/observable/zip';

@Component({
  selector: 'app-organization-list',
  templateUrl: './organization-list.component.html',
  styles: [],
  providers: [AlarmConfigService]
})
export class OrganizationListComponent implements OnInit {
  private organization: Organization;
  grid: Grid<Organization> = new Grid(null);
  queryMap = { text: '请输入名称', value: ''};
  queryTextStream: Subject<string> = new Subject<string>();
    private initPage() {
    this.organization =  {
      name: {
        text: '名称',
        width: '120px'  
      },
      rank: {
        text: '级别',
        width: '80px',
        format: (value: any, col: Column, row: any) => {
          const item = [
            { value: 0, label: '企业' },
            { value: 1, label: '乡镇科级' },
            { value: 2, label: '县处级' },
            { value: 3, label: '司厅局级' },
            { value: 4, label: '省部级' },
            { value: 5, label: '国家级' }
          ].filter( (rankItem: {value: number, label: string }) => {
            return rankItem.value === value;
          });
          return item != null && item.length === 1 ? item[0].label : '';
        }
      },
      telephone: {
        text: '电话',
        width: '120px'
      },
      address: {
        text: '地址',
        width: '300px',
        format: (value: any, col: Column, row: any) => {
              value = value == null ? '' : value ;
              if (row['areaNames'] != null) {
                return row['areaNames']['provinceName'] + row['areaNames']['cityName'] + row['areaNames']['areaName'] + value;
              } else {
                return value;
              }
              
        }
      },
      createTime: {
        text: '创建时间',
        width: '100px',
        type: {
         name: Types.Date,
         format: 'YYYY-MM-DD HH:mm:ss'
        },
         isSort: true
      },
      expireTime: {
        text: '到期时间',
        width: '100px',
        type: {
         name: Types.Date,
         format: 'YYYY-MM-DD HH:mm:ss'
        },
         isSort: true  
      }
    };
    this.grid.title = '组织';
    this.grid.setColumns(this.organization);
    this.grid.pageSize = 10;
  }
  constructor(
    private organizationService: OrganizationService,
    
    private confirmServ: NzModalService,
    public msgSrv: NzMessageService,
    private modalHelper: ModalHelper,
    private router: Router,
    private sensorsService: SensorsService,
    private alarmConfigService: AlarmConfigService,
    private http: _HttpClient
  ) {}

  ngOnInit() {
    this.initPage();
    this.queryTextStream
    .debounceTime(500)
    .distinctUntilChanged()
    .subscribe(queryText => {
        this.load();
    });
  }
  queryTextChanged($event) {
      this.queryTextStream.next(this.queryMap.value);
  }
  load(reload: boolean = false) {   
    if (reload) {
      this.grid.pageIndex = 1 ;
    }
    // 延时加载避免ExpressionChangedAfterItHasBeenCheckedError
    setTimeout(() => {
      this.grid.loading = true;
     }, 1);
    this.organizationService.getPagingList(this.grid, this.queryMap.value).subscribe(
       (res: PageBean) => {
            this.grid.loading = true;
            if (res != null && res.data != null) {
              this.grid.initData(res);
              this.grid.refreshStatus();
              setTimeout(() => {
                this.grid.loading = false;
               }, 1);
            }
       }
    );
  }

// rowData为null时，为新增
  addOrModify(d) {
    const data = {};
    if ( d != null) {
      Object.assign(data, d);
    }
    const cols = this.organization;
    this.modalHelper.static(OrganizationEditComponent, { cols , data }).subscribe( 
      ( ret: { data: any, close: Function} ) => {
      // 修改状态
      if (ret.data['index'] != null ) {
          const index: number = ret.data['index'] ;
          const origData = this.grid.getData()[index];
          const isModified =  Object.keys(origData).some(
            (key: string) => {
                return ret.data[key] !== origData[key];    
            } 
          );
          // 未作修改
          if (!isModified) { 
            ret.close();
            this.msgSrv.success('组织未作任何修改！');
            return;
          }
      }
      this.organizationService.save(ret.data).subscribe(
         ( res: any) => {
            if (res.code === 1) {
              this.load();
              ret.close();
              this.msgSrv.success('组织保存成功！');
            }
         }
      );
    });
  }

  delete(...id: number[]) {
    this.organizationService.delete( ...id ).subscribe(
      ( res: any) => {
         if (res.code === 1) {
           this.load();
           this.msgSrv.success('设备型号删除成功！');
         }
      }
   );
  }

   deleteSelected() {
    this.confirmServ.confirm({
      title: '批量删除',
      content: '注意：数据一旦删除，将不可恢复！',
      okText: '确定',
      cancelText: '取消'
    }).on('onOk', () => {
       if (this.grid.selectedIndexs != null && this.grid.selectedIndexs.length > 0) {
          const ids = this.grid.selectedIndexs.map(
              (index: number) => {
                  const id = this.grid.data[index].id;
                  return Number.parseInt(id);
              }
          );
          this.delete( ...ids );
       }
    });
  }
  sort(field: string, value: string) {
    // 删除当前field
    this.grid.sorts = this.grid.sorts.filter(
       (fn: string) => {
          return fn !== field;
       }
     );
     // 如果value不为null，在排序数组最后加上filed
     if ( value != null ) {
        this.grid.sorts.push(field);
     }
     this.load();
  }
   configAlarm(row) {
        // 延时加载避免ExpressionChangedAfterItHasBeenCheckedError
        setTimeout(() => {
          this.grid.loading = true;
         }, 1);
    zip(
      this.sensorsService.getPagingList({pageIndex: 0, pageSize: 0}, null),
      this.alarmConfigService.getByOid( row.id )
    ).subscribe(([pageBean, resultBean]) => {
      this.grid.loading = false;
      this.organizationService.handle = 'config' ;
      this.organizationService.data = row;
      this.organizationService.config = {pageBean, resultBean};
      this.organizationService.title = '配置三级警报';
    });
  }
    configScreenLayout(row) {
              // 延时加载避免ExpressionChangedAfterItHasBeenCheckedError
              setTimeout(() => {
                this.grid.loading = true;
                }, 1);
              this.http.get('org-layout/rtd-config', {orgId: row.id}).subscribe(
                   (res: ResultBean<any>) => {
                      if ( res.code > 0 ) {
                        this.grid.loading = false;
                        this.organizationService.handle = 'screen' ;
                        this.organizationService.data = row;
                        this.organizationService.config['resultBean'] = res;
                        this.organizationService.title = '配置大屏布局';
                      }
                   }
              );
    }
    configShowUnit(row) {
        // 延时加载避免ExpressionChangedAfterItHasBeenCheckedError
        setTimeout(() => {
          this.grid.loading = true;
          }, 1);
        this.http.get('org-sunit/gets-byoid', {orgId: row.id}).subscribe(
             (res: ResultBean<any>) => {
                if ( res.code > 0 ) {
                  this.grid.loading = false;
                  this.organizationService.handle = 'unit' ;
                  this.organizationService.data = row;
                  this.organizationService.config['resultBean'] = res;
                  this.organizationService.title = '配置显示单位';
                }
             }
        );
    }
  registration(row) {
      sessionStorage.setItem('organization', JSON.stringify(row));
  }
}
